
c This program is to read an ASCII .psf file created by running
c standalone DAOPHOT and write an output fits image of the PSF.
c I haven't figured out a way to do that with the IRAF daophot
c tasks.

c This is useful if you're doing something like reducing MIPS data
c with DAOPHOT and constructing an empirical PSF from your data.

c Note that DAOPHOT tabulates the PSF on pixels that are half the
c size of your data pixels (I believe so, see Stetson 1987)

      program dao_makepsf

      parameter (MAXARR=1024)

      real dataline(6)
      real data1d(MAXARR*MAXARR),data2d(MAXARR,MAXARR)
      real psf2d(MAXARR,MAXARR)
      integer ioutsize(2),fpixels(2),lpixels(2)
      
      character cdata*120, psftype*9
      character infile*120,outimage*120

      iarrsize = MAXARR
      root2pi = sqrt(2.0 * 3.14159265)

c Number of values DAOPHOT writes on one line
      nperline = 6

 100  continue
      write(*,'("Input psf file: ",$)')
      read(*,'(a)') infile
      open(2,err=100,file=infile,status='old')
      
 110  continue
      write(*,'("Name for output psf image: ",$)')
      read(*,'(a)') outimage
      
c fitsio file open
      ierr=0
      call ftgiou(imout1,ierr)
      call ftinit(imout1,outimage,iblock,ierr)
      if (ierr .ne. 0) then 
         write(*,*) "Couldnt open output image! ",outimage
         call doerr(ierr)
         go to 110
      end if

      read(2,'(a)') cdata
      read(cdata(1:9),'(a9)') psftype
      if (psftype(1:9) .ne. ' GAUSSIAN') then
         write(*,'(a,a)') "PSF file says not GAUSSIAN but ",psftype
         write(*,*) 
     $        "Stopping because program only set up to do gaussians"
         go to 666
      end if
      
c need to figure out what the params are.  The lines in DAOPHOT
c source in psf.f that write this are:
c      WRITE (3,202) LABEL, NPSF, NPAR, NTAB(IEXPAND), 5*IFRAC,
c     .     PSFMAG, SCALE, XMID, YMID
c  202 FORMAT (1X, A8, 4I5, F9.3, F15.3, 2F9.1)
c      WRITE (3,201) (PAR(L), L=1,NPAR)
c  201       FORMAT (1X, 1P, 6E13.6)
c the actual PSF residuals are printed with
c         DO K=1,NEXP
c            WRITE (3,204) ((PSF(I,J,K), I=1,NPSF), J=1,NPSF)
c  204       FORMAT (1X, 1P, 6E13.6)
c         END DO


c par(1) and par(2) are obviously a FWHM or sigma, but which
c and are they in full or half pixels?  For MIPS 24 data on 1.245"
c pixels these are = 2.32, which makes me think they are 
c sigma in units of half-pixels.  Although that gives a FWHM
c of 3.4 arcsec and it should come out closer to 6".

c Lei thinks these may be HWHM in units of original pixels,
c in which case it implies a FWHM of 5.8" which is better.
c However, making a PSF this way, it is not very strongly peaked,
c so maybe I have the normalization or interpretation of "scale"
c parameter wrong.

c For MIPS 70 data on 4.925" pixels, par(1) = 1.96, expected FWHM ~ 18"
c for MIPS 160 on 8.0" pixels, par(1) = 2.58, expected FWHM ~ 40"

c so par(1)*originalpixelsize / FWHM-in-arcsec ~ 0.48, 0.54, 0.52
c this supports the idea that par(1) is HWHM in original pixels,
c and we need to figure out what "scale" is to normalize the
c Gaussian part of the PSF correctly relative to the residual array.

c Last page of the DAOphot manual appears to say that scale is the
c height of the center of the analytic part of the PSF, not the
c integral of it.  And that the widths are HWHM (it doesn't say
c in what, but must be original pixels)

      read(cdata(10:),*) nrow,npar,ntab,ifrac,psfmag,scale,xmid,ymid
      read(2,*) xrad,yrad
      write(*,*) nrow, scale, xrad,yrad

c if par(1) and par(2) are sigma in units of half-pixels
      xsig = xrad
      ysig = yrad
c if par(1) and par(2) are HWHM in units of original pixels
c convert to sigma in units of half-pixels
      xsig = xrad*2.0 / 2.35 * 2.0
      ysig = yrad*2.0 / 2.35 * 2.0

      xsigsq2 = 2.0 * xsig**2
      ysigsq2 = 2.0 * ysig**2
      r2pisigs = root2pi * xsig * ysig

c if we want an analytic part to have peak height = scale
c rather than to integrate to 1.  Do we have to divide by 4
c to conserve flux because we're calculating on half-pixels?
c still seems to be about 1.4x too high.  dividing by 1.4 further
c is just a fudge based on comparing to height of peak to 
c first airy ring in some data.
      scale = scale * r2pisigs / 4.0 / 1.4
      
      nptot = nrow*nrow
c read all the full lines
c Can't read free-format because there is no space in front of numbers
c with a minus sign
      nfulllines = int(nptot/nperline)
      do iline=1,nfulllines
         ioffset = (iline-1) * nperline
         read(2,*,err=500,end=500) (data1d(ioffset+i),i=1,nperline)
c         read(2,1001,err=500,end=500) (data1d(ioffset+i),i=1,nperline)
      end do
 1001 format(1X, 1P, 6E13.6)

c read the last partial line
      idone = nfulllines*nperline
      nleft = nptot-idone
      read(2,*,err=500,end=500) (data1d(idone+i),i=1,nleft)
c      read(2,1001,err=500,end=500) (data1d(idone+i),i=1,nleft)
c      read(2,1001) (data1d(idone+i),i=1,nleft)

c reformat data into 2-d array
      sum = 0.0
      do j=1,nrow
         ioffset = (j-1)*nrow
         do i=1,nrow
            data2d(i,j) = data1d(ioffset+i)
            sum = sum + data1d(ioffset+i)
         end do
      end do
      write(*,*) "Mean of resid array: ",sum/nrow**2
      
c go through data adding in the Gaussian
      xcen = (nrow+1)/2.0
      ycen = (nrow+1)/2.0
      do j=1,nrow
         ydist = j-ycen
         ioffset = (j-1)*nrow
         do i=1,nrow
            xdist = i-xcen
            psfcoreval = scale / r2pisigs * 
     $           exp(-(xdist**2/xsigsq2)-(ydist**2/ysigsq2))
            psf2d(i,j) = psfcoreval + data2d(i,j)
            i1d = ioffset + i
            data1d(i1d) = psfcoreval + data1d(i1d)
         end do
      end do

      open(3,file='dao_makepsf.out',status='unknown')
      nnn = 0
      do j=1,nrow
         do i=1,nrow
            nnn = nnn+1
            write(3,*) nnn,j,i,data2d(i,j),psf2d(i,j)
         end do
      end do
      close(3)

c write a FITS image
      ioutdim = 2
      ioutsize(1) = nrow
      ioutsize(2) = nrow
      ibitpix=-32
c put header values
      call ftphps(imout1,ibitpix,ioutdim,ioutsize,ierr)
      call doerr(ierr)
      fpixels(1) = 1
      fpixels(2) = 1
      lpixels(1) = nrow
      lpixels(2) = nrow
c Try just writing from the 1-d array.  This appears to work
      call ftppre(imout1,igroup,1,nrow*nrow,data1d,ierr)
c One of these should write from the 2-d array but it isn't working
c      call ftpsse(imout1,igroup,ioutdim,ioutsize,fpixels,lpixels,
c     $     psf2d,ierr)
c      call ftp2de(imout1,igroup,iarrsize,ioutsize(1),ioutsize(2),
c     $     psf2d,ierr)
      call doerr(ierr)

 400  continue
c success - skip to end
      go to 666

 500  continue
      write(*,*) "Ran out of data in input file ",nptot,iline,ioffset
      go to 666

 666  continue
      close(2)
      
c fitsio close
      call ftclos(imout1,ierr)
      call doerr(ierr)
      call ftfiou(imout1,ierr)
      call doerr(ierr)

      end


cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine doerr( status)

      integer status
      character*80 error

      if ( status.ne.0) then
      call ftgerr( status, error)
      write(6,25) error
c  25      format(20x,a30)
  25      format(2x,'fitsio: ',a30)
      end if

      status = 0

      call ftcmsg()

      return
      end

